@@ -42,7 +42,6 @@
 #include <gdk/gdkkeysyms.h>
 #include <X11/keysym.h>
 #include <X11/Xatom.h>
-#include <X11/extensions/XInput2.h>
 
 #include <gdesktop-enums.h>
 
@@ -68,7 +67,7 @@
 #define KEY_SPEED               "speed"
 
 /* Touchpad settings */
-#define KEY_SCROLL_METHOD                "scroll-method"
+#define KEY_EDGE_SCROLLING_ENABLED       "edge-scrolling-enabled"
 #define KEY_TAP_TO_CLICK                 "tap-to-click"
 #define KEY_SEND_EVENTS                  "send-events"
 #define KEY_NATURAL_SCROLL_ENABLED       "natural-scroll"
@@ -338,22 +337,23 @@ bail:
 }
 
 static gboolean
-touchpad_has_single_button (int deviceid)
+touchpad_has_single_button (XDevice *device)
 {
-        GdkDisplay *display = gdk_display_get_default ();
-        Atom type;
+        Atom type, prop;
         int format;
         unsigned long nitems, bytes_after;
         unsigned char *data;
         gboolean is_single_button = FALSE;
         int rc;
 
+        prop = XInternAtom (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), "Synaptics Capabilities", False);
+        if (!prop)
+                return FALSE;
+
         gdk_error_trap_push ();
-
-        rc = XIGetProperty (GDK_DISPLAY_XDISPLAY (display), deviceid,
-                            gdk_x11_get_xatom_by_name ("Synaptics Capabilities"),
-                            0, 1, False, XA_INTEGER, &type, &format, &nitems, &bytes_after, &data);
-
+        rc = XGetDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device, prop, 0, 1, False,
+                                XA_INTEGER, &type, &format, &nitems,
+                                &bytes_after, &data);
         if (rc == Success && type == XA_INTEGER && format == 8 && nitems >= 3)
                 is_single_button = (data[0] == 1 && data[1] == 0 && data[2] == 0);
 
@@ -383,13 +383,19 @@ set_left_handed (GsdMouseManager *manager,
         if (xdevice_is_libinput (gdk_x11_device_get_id (device)))
                 return;
 
+        xdevice = open_gdk_device (device);
+        if (xdevice == NULL)
+                return;
+
 	g_debug ("setting handedness on %s", gdk_device_get_name (device));
 
+        buttons = g_new (guchar, buttons_capacity);
+
         /* If the device is a touchpad, swap tap buttons
          * around too, otherwise a tap would be a right-click */
-        if (device_is_synaptics (gdk_x11_device_get_id (device))) {
+        if (xdevice_is_synaptics (xdevice)) {
                 gboolean tap = g_settings_get_boolean (manager->priv->touchpad_settings, KEY_TAP_TO_CLICK);
-                gboolean single_button = touchpad_has_single_button (gdk_x11_device_get_id (device));
+                gboolean single_button = touchpad_has_single_button (xdevice);
 
                 left_handed = touchpad_left_handed;
 
@@ -397,18 +403,13 @@ set_left_handed (GsdMouseManager *manager,
                         set_tap_to_click (device, tap, left_handed);
 
                 if (single_button)
-                        return;
+                        goto out;
         } else {
                 left_handed = mouse_left_handed;
         }
 
-        xdevice = open_gdk_device (device);
-        if (xdevice == NULL)
-                return;
-
         gdk_error_trap_push ();
 
-        buttons = g_new (guchar, buttons_capacity);
         n_buttons = XGetDeviceButtonMapping (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice,
                                              buttons,
                                              buttons_capacity);
@@ -428,6 +429,7 @@ set_left_handed (GsdMouseManager *manager,
         XSetDeviceButtonMapping (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice, buttons, n_buttons);
         gdk_error_trap_pop_ignored ();
 
+out:
         xdevice_close (xdevice);
         g_free (buttons);
 }
@@ -456,7 +458,7 @@ set_motion (GsdMouseManager *manager,
 
 	g_debug ("setting motion on %s", gdk_device_get_name (device));
 
-        if (device_is_synaptics (gdk_x11_device_get_id (device)))
+        if (xdevice_is_synaptics (xdevice))
                 settings = manager->priv->touchpad_settings;
         else
                 settings = manager->priv->mouse_settings;
@@ -628,8 +630,6 @@ set_tap_to_click (GdkDevice *device,
 
         if (xdevice_is_libinput (gdk_x11_device_get_id (device)))
                 return;
-        if (!device_is_synaptics (gdk_x11_device_get_id (device)))
-                return;
 
         prop = XInternAtom (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), "Synaptics Tap Action", False);
         prop_capabilities = XInternAtom (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), "Synaptics Capabilities", False);
@@ -640,6 +640,11 @@ set_tap_to_click (GdkDevice *device,
         if (xdevice == NULL)
                 return;
 
+        if (!xdevice_is_synaptics (xdevice)) {
+                xdevice_close (xdevice);
+                return;
+        }
+
 	g_debug ("setting tap to click on %s", gdk_device_get_name (device));
 
         gdk_error_trap_push ();
@@ -689,8 +694,6 @@ set_horiz_scroll (GdkDevice *device,
 
         if (xdevice_is_libinput (gdk_x11_device_get_id (device)))
                 return;
-        if (!device_is_synaptics (gdk_x11_device_get_id (device)))
-                return;
 
         prop_edge = XInternAtom (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), "Synaptics Edge Scrolling", False);
         prop_twofinger = XInternAtom (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), "Synaptics Two-Finger Scrolling", False);
@@ -702,6 +705,11 @@ set_horiz_scroll (GdkDevice *device,
         if (xdevice == NULL)
                 return;
 
+        if (!xdevice_is_synaptics (xdevice)) {
+                xdevice_close (xdevice);
+                return;
+        }
+
 	g_debug ("setting horiz scroll on %s", gdk_device_get_name (device));
 
         gdk_error_trap_push ();
@@ -742,9 +750,9 @@ set_horiz_scroll (GdkDevice *device,
 }
 
 static void
-set_scroll_method (GsdMouseManager         *manager,
-                   GdkDevice               *device,
-                   GsdTouchpadScrollMethod  method)
+set_edge_scrolling_enabled (GsdMouseManager         *manager,
+                            GdkDevice               *device,
+                            gboolean                 enabled)
 {
         int rc;
         XDevice *xdevice;
@@ -752,11 +760,10 @@ set_scroll_method (GsdMouseManager         *manager,
         int act_format;
         unsigned long nitems, bytes_after;
         unsigned char *data;
+        GsdTouchpadScrollMethod method;
 
         if (xdevice_is_libinput (gdk_x11_device_get_id (device)))
                 return;
-        if (!device_is_synaptics (gdk_x11_device_get_id (device)))
-                return;
 
         prop = XInternAtom (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), "Synaptics Capabilities", True);
         prop_edge = XInternAtom (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), "Synaptics Edge Scrolling", False);
@@ -769,6 +776,11 @@ set_scroll_method (GsdMouseManager         *manager,
         if (xdevice == NULL)
                 return;
 
+        if (!xdevice_is_synaptics (xdevice)) {
+                xdevice_close (xdevice);
+                return;
+        }
+
 	g_debug ("setting edge scroll on %s", gdk_device_get_name (device));
 
         gdk_error_trap_push ();
@@ -777,14 +789,18 @@ set_scroll_method (GsdMouseManager         *manager,
                                  XA_INTEGER, &act_type, &act_format, &nitems,
                                  &bytes_after, &data);
         if (rc == Success && act_type != None) {
-                if (!(data[3]) && method == GSD_TOUCHPAD_SCROLL_METHOD_TWO_FINGER_SCROLLING) {
-                        g_warning ("Two finger scroll is not supported by %s", gdk_device_get_name (device));
-                        method = GSD_TOUCHPAD_SCROLL_METHOD_EDGE_SCROLLING;
-                }
+                /* Two-finger scrolling is supported, so enable it */
+                if (data[3])
+                        method = GSD_TOUCHPAD_SCROLL_METHOD_TWO_FINGER_SCROLLING;
 
                 XFree (data);
         }
 
+        if (enabled && method != GSD_TOUCHPAD_SCROLL_METHOD_TWO_FINGER_SCROLLING)
+                method = GSD_TOUCHPAD_SCROLL_METHOD_EDGE_SCROLLING;
+        else
+                method = GSD_TOUCHPAD_SCROLL_METHOD_DISABLED;
+
         rc = XGetDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice,
                                  prop_edge, 0, 1, False,
                                  XA_INTEGER, &act_type, &act_format, &nitems,
@@ -828,8 +844,6 @@ set_touchpad_disabled (GdkDevice *device)
 
         if (xdevice_is_libinput (gdk_x11_device_get_id (device)))
                 return;
-        if (!device_is_synaptics (gdk_x11_device_get_id (device)))
-                return;
 
         g_object_get (G_OBJECT (device), "device-id", &id, NULL);
 
@@ -839,6 +853,11 @@ set_touchpad_disabled (GdkDevice *device)
         if (xdevice == NULL)
                 return;
 
+        if (!xdevice_is_synaptics (xdevice)) {
+                xdevice_close (xdevice);
+                return;
+        }
+
         if (set_synaptics_device_enabled (id, FALSE) == FALSE)
                 g_warning ("Error disabling device \"%s\" (%d)", gdk_device_get_name (device), id);
         else
@@ -854,8 +873,6 @@ set_touchpad_enabled (int id)
 
         if (xdevice_is_libinput (id))
                 return;
-        if (!device_is_synaptics (id))
-                return;
 
         g_debug ("Trying to set device enabled for %d", id);
 
@@ -864,6 +881,11 @@ set_touchpad_enabled (int id)
         if (gdk_error_trap_pop () != 0)
                 return;
 
+        if (!xdevice_is_synaptics (xdevice)) {
+                xdevice_close (xdevice);
+                return;
+        }
+
         if (set_synaptics_device_enabled (id, TRUE) == FALSE)
                 g_warning ("Error enabling device \"%d\"", id);
         else
@@ -966,15 +988,18 @@ set_natural_scroll (GsdMouseManager *manager,
         unsigned char *data;
         glong *ptr;
 
-        if (xdevice_is_libinput (gdk_x11_device_get_id (device)))
-                return;
-        if (!device_is_synaptics (gdk_x11_device_get_id (device)))
-                return;
-
         xdevice = open_gdk_device (device);
         if (xdevice == NULL)
                 return;
 
+        if (!xdevice_is_synaptics (xdevice)) {
+                xdevice_close (xdevice);
+                return;
+        }
+
+        if (xdevice_is_libinput (gdk_x11_device_get_id (device)))
+                return;
+
         g_debug ("Trying to set %s for \"%s\"",
                  natural_scroll ? "natural (reverse) scroll" : "normal scroll",
                  gdk_device_get_name (device));
@@ -1126,7 +1151,7 @@ set_mouse_settings (GsdMouseManager *manager,
         set_motion (manager, device);
 
         set_tap_to_click (device, g_settings_get_boolean (manager->priv->touchpad_settings, KEY_TAP_TO_CLICK), touchpad_left_handed);
-        set_scroll_method (manager, device, g_settings_get_enum (manager->priv->touchpad_settings, KEY_SCROLL_METHOD));
+        set_edge_scrolling_enabled (manager, device, g_settings_get_boolean (manager->priv->touchpad_settings, KEY_EDGE_SCROLLING_ENABLED));
         set_horiz_scroll (device, TRUE);
         set_natural_scroll (manager, device, g_settings_get_boolean (manager->priv->touchpad_settings, KEY_NATURAL_SCROLL_ENABLED));
 
@@ -1196,8 +1221,8 @@ touchpad_callback (GSettings       *settings,
                         mouse_left_handed = g_settings_get_boolean (manager->priv->mouse_settings, KEY_LEFT_HANDED);
                         set_tap_to_click (device, g_settings_get_boolean (settings, key),
                                           get_touchpad_handedness (manager, mouse_left_handed));
-                } else if (g_str_equal (key, KEY_SCROLL_METHOD)) {
-                        set_scroll_method (manager, device, g_settings_get_enum (settings, key));
+                } else if (g_str_equal (key, KEY_EDGE_SCROLLING_ENABLED)) {
+                        set_edge_scrolling_enabled (manager, device, g_settings_get_boolean (settings, key));
                         set_horiz_scroll (device, TRUE);
                 } else if (g_str_equal (key, KEY_SPEED)) {
                         set_motion (manager, device);
@@ -1498,6 +1523,18 @@ map_send_events (GVariant *variant)
         }
 }
 
+static GVariant *
+map_edge_scrolling_enabled (GVariant *variant)
+{
+	GsdTouchpadScrollMethod  method;
+
+	method = g_variant_get_uint32 (variant);
+	if (method == GSD_TOUCHPAD_SCROLL_METHOD_EDGE_SCROLLING)
+		return g_variant_new_boolean (TRUE);
+	else
+		return g_variant_new_boolean (FALSE);
+}
+
 static void
 migrate_mouse_settings (void)
 {
@@ -1513,7 +1550,7 @@ migrate_mouse_settings (void)
         GsdSettingsMigrateEntry touchpad_entries[] = {
                 { "disable-while-typing", NULL,             NULL },
                 { "horiz-scroll-enabled", NULL,             NULL },
-                { "scroll-method",        "scroll-method",  NULL },
+                { "scroll-method",        "edge-scrolling-enabled", map_edge_scrolling_enabled },
                 { "tap-to-click",         "tap-to-click",   NULL },
                 { "touchpad-enabled",     "send-events",    map_send_events },
                 { "left-handed",          "left-handed",    NULL },
